"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteCollection = exports.slashPathToFirestoreRef = exports.isDocPath = exports.initializeFirebase = exports.isString = void 0;
var tslib_1 = require("tslib");
var node_utils_1 = require("./node-utils");
/**
 * Check whether a value is a string or not
 * @param valToCheck - Value to check
 * @returns Whether or not value is a string
 */
function isString(valToCheck) {
    return typeof valToCheck === 'string' || valToCheck instanceof String;
}
exports.isString = isString;
/**
 * Get settings for Firestore from environment. Loads port and servicePath from
 * FIRESTORE_EMULATOR_HOST node environment variable if found, otherwise
 * defaults to port 8080 and servicePath "localhost".
 * @returns Firestore settings to be passed to firebase.firestore().settings
 */
function firestoreSettingsFromEnv() {
    var FIRESTORE_EMULATOR_HOST = process.env.FIRESTORE_EMULATOR_HOST;
    if (typeof FIRESTORE_EMULATOR_HOST === 'undefined' ||
        !isString(FIRESTORE_EMULATOR_HOST)) {
        return {
            servicePath: 'localhost',
            port: 8080,
        };
    }
    var _a = tslib_1.__read(FIRESTORE_EMULATOR_HOST.split(':'), 2), servicePath = _a[0], portStr = _a[1];
    return {
        servicePath: servicePath,
        port: parseInt(portStr, 10),
    };
}
var fbInstance;
/**
 * Initialize Firebase instance from service account (from either local
 * serviceAccount.json or environment variables)
 * @returns Initialized Firebase instance
 * @param adminInstance - firebase-admin instance to initialize
 * @param overrideConfig - firebase-admin instance to initialize
 */
function initializeFirebase(adminInstance, overrideConfig) {
    if (fbInstance) {
        return fbInstance;
    }
    try {
        // TODO: Look into using @firebase/testing in place of admin here to allow for
        // usage of clearFirestoreData (see https://github.com/prescottprue/cypress-firebase/issues/73 for more info)
        var serviceAccount = node_utils_1.getServiceAccount();
        var projectId = process.env.GCLOUD_PROJECT ||
            process.env.FIREBASE_PROJECT ||
            process.env.FIREBASE_PROJECT_ID ||
            (serviceAccount === null || serviceAccount === void 0 ? void 0 : serviceAccount.project_id); // eslint-disable-line camelcase
        var FIREBASE_DATABASE_EMULATOR_HOST = process.env.FIREBASE_DATABASE_EMULATOR_HOST;
        var fbConfig = tslib_1.__assign({ projectId: projectId }, overrideConfig);
        if (FIREBASE_DATABASE_EMULATOR_HOST) {
            // Initialize RTDB with databaseURL pointed to emulator if FIREBASE_DATABASE_EMULATOR_HOST is set
            fbConfig.databaseURL = "http://" + FIREBASE_DATABASE_EMULATOR_HOST + "?ns=" + (projectId || 'local');
            /* eslint-disable no-console */
            console.log('cypress-firebase: Using RTDB emulator with DB URL:', fbConfig.databaseURL);
            /* eslint-enable no-console */
        }
        if (process.env.FIREBASE_AUTH_EMULATOR_HOST) {
            /* eslint-disable no-console */
            console.log('cypress-firebase: Using Auth emulator with port:', process.env.FIREBASE_AUTH_EMULATOR_HOST);
            /* eslint-enable no-console */
        }
        // Add service account credential if it exists so that custom auth tokens can be generated
        if (serviceAccount) {
            fbConfig.credential = adminInstance.credential.cert(serviceAccount);
        }
        fbInstance = adminInstance.initializeApp(fbConfig);
        // Initialize Firestore with emulator host settings
        if (process.env.FIRESTORE_EMULATOR_HOST) {
            var firestoreSettings = firestoreSettingsFromEnv();
            /* eslint-disable no-console */
            console.log('cypress-firebase: Using Firestore emulator with settings:', firestoreSettings);
            /* eslint-enable no-console */
            adminInstance.firestore().settings(firestoreSettings);
        }
        /* eslint-disable no-console */
        console.log("cypress-firebase: Initialized app with database url \"" + fbConfig.databaseURL + "\"");
        /* eslint-enable no-console */
        return fbInstance;
    }
    catch (err) {
        /* eslint-disable no-console */
        console.error('cypress-firebase: Error initializing firebase-admin instance:', err.message);
        /* eslint-enable no-console */
        throw err;
    }
}
exports.initializeFirebase = initializeFirebase;
/**
 * Check with or not a slash path is the path of a document
 * @param slashPath - Path to check for whether or not it is a doc
 * @returns Whether or not slash path is a document path
 */
function isDocPath(slashPath) {
    return !(slashPath.replace(/^\/|\/$/g, '').split('/').length % 2);
}
exports.isDocPath = isDocPath;
/**
 * Convert slash path to Firestore reference
 * @param firestoreInstance - Instance on which to
 * create ref
 * @param slashPath - Path to convert into firestore refernce
 * @param options - Options object
 * @returns Ref at slash path
 */
function slashPathToFirestoreRef(firestoreInstance, slashPath, options) {
    var _a;
    if (!slashPath) {
        throw new Error('Path is required to make Firestore Reference');
    }
    var ref = isDocPath(slashPath)
        ? firestoreInstance.doc(slashPath)
        : firestoreInstance.collection(slashPath);
    // Apply orderBy to query if it exists
    if ((options === null || options === void 0 ? void 0 : options.orderBy) && typeof ref.orderBy === 'function') {
        if (Array.isArray(options.orderBy)) {
            ref = ref.orderBy.apply(ref, tslib_1.__spreadArray([], tslib_1.__read(options.orderBy)));
        }
        else {
            ref = ref.orderBy(options.orderBy);
        }
    }
    // Apply where to query if it exists
    if ((options === null || options === void 0 ? void 0 : options.where) &&
        Array.isArray(options.where) &&
        typeof ref.where === 'function') {
        if (Array.isArray(options.where[0])) {
            ref = (_a = ref.where.apply(ref, tslib_1.__spreadArray([], tslib_1.__read(options.where[0])))).where.apply(_a, tslib_1.__spreadArray([], tslib_1.__read(options.where[1])));
        }
        else {
            ref = ref.where.apply(ref, tslib_1.__spreadArray([], tslib_1.__read(options.where)));
        }
    }
    // Apply limit to query if it exists
    if ((options === null || options === void 0 ? void 0 : options.limit) && typeof ref.limit === 'function') {
        ref = ref.limit(options.limit);
    }
    // Apply limitToLast to query if it exists
    if ((options === null || options === void 0 ? void 0 : options.limitToLast) && typeof ref.limitToLast === 'function') {
        ref = ref.limitToLast(options.limitToLast);
    }
    return ref;
}
exports.slashPathToFirestoreRef = slashPathToFirestoreRef;
/**
 * @param db - Firestore instance
 * @param query - Query which is limited to batch size
 * @param resolve - Resolve function
 * @param reject - Reject function
 */
function deleteQueryBatch(db, query, resolve, reject) {
    query
        .get()
        .then(function (snapshot) {
        // When there are no documents left, we are done
        if (snapshot.size === 0) {
            return 0;
        }
        // Delete documents in a batch
        var batch = db.batch();
        snapshot.docs.forEach(function (doc) {
            batch.delete(doc.ref);
        });
        return batch.commit().then(function () { return snapshot.size; });
    })
        .then(function (numDeleted) {
        if (numDeleted === 0) {
            resolve();
            return;
        }
        // Recurse on the next process tick, to avoid
        // exploding the stack.
        process.nextTick(function () {
            deleteQueryBatch(db, query, resolve, reject);
        });
    })
        .catch(reject);
}
/**
 * @param db - Firestore instance
 * @param collectionPath - Path of collection
 * @param batchSize - Size of delete batch
 * @returns Promise which resolves with results of deleting batch
 */
function deleteCollection(db, collectionPath, batchSize) {
    var collectionRef = db.collection(collectionPath);
    var query = collectionRef.orderBy('__name__').limit(batchSize || 500);
    return new Promise(function (resolve, reject) {
        deleteQueryBatch(db, query, resolve, reject);
    });
}
exports.deleteCollection = deleteCollection;
